from setuptools import find_packages, setup

setup(
    name='okgrade',
    version='0.4.1',
    license='3-clause BSD',
    author='Yuvi Panda',
    author_email='yuvipanda@gmail.com',
    description='Simple alternative to okpy for grading only',
    packages=find_packages()
)
